# mtk.awk
#
# Mini-ToolKit v 2.0
# (c) 1989 Jon Berlin
#
# for Searchlight bulletin board systems


# body of program - interprets mode of operation and calls functions
{
   {
   if (c == "N" || c == "C" || c == "M" && x != 1)
       color()
       }
   if (M == "NU")  {
       newuser()
       }
   else if (M == "FR")  {
       report()
       }
   else if (M == "FT")  {
       files_transfers()
       }
   else if (M == "UT")  {
       usertime()
       }
   else if (M == "PF")  {
       popular_files()
       }
   else if (M == "RL")  {
       relister()
   }
}

function report()

# function to generate alphabetical list of users and the files they
# have uploaded and downloaded.
# syntax:  mtk M=FR files.log
{
   ftrans_flag = 1
   ut_flag = 1

   person = substr($0,30)
   type = substr($3,1,1)
   if ((person,type) in names)           # generate array
      names[person,type] = names[person,type] SUBSEP $4
   else
      names[person,type] = $4
}
END   {
      for (i in names)
      {
      split(i,elements,SUBSEP)
      if (elements[1] != starter)  {
         print
         print elements[1]
         starter = elements[1]
      }
      nfiles = split(names[i],files,SUBSEP)
      if (elements[2] =="D")
         print "  Downloads:"
      else
         print "  Uploads:"

      for (k = 1;k <=nfiles; k++)
         print "    " files[k]
      }
}


function newuser()

# function to generate list of new users.
# syntax:  mtk M=NU c=%G activity.log
{
   ut_flag = 1
   ftrans_flag = 1
   if ($5 == "*")
      printf("\n%s %s %s %s %s", $6, $7, $8, $9, $10)
}
END   {
      if (newuser_flag != 1) {
      printf("\n\n")
      printf("%s", DEFAULT)
      }
}


function files_transfers()

# function to list a single users file transfers - either up or down
# syntax:  mtk M=FT u="%N" s=D (or U for upload) c=%G files.log
{
   ut_flag = 1
   newuser_flag = 1

   if (uod != 1)  {
      up_down()
   }

   if (($3 == s) && ($0 ~ u))
      {
      count++
      printf("%15s  %s\n", $4, $1) }
      }

END   {
      if (count == 0 && ftrans_flag != 1)
         printf("\nYou haven't %s any files.%s\n", up_or_down, DEFAULT)
      else if (ftrans_flag != 1)
         printf("\nYou have %s %-2d file(s).%s\n\n", up_or_down, count, DEFAULT)
}


function up_down()

# function called by files_transfers() to generate header information
{
   if (s == "D") {
      s = "D/L"
      up_or_down = "downloaded"
   }
   else {
      s = "U/L"
      up_or_down = "uploaded"
   }
   printf("Files you have %s.\n\n", up_or_down)
   printf("       Filename   Date\n")
   printf("          \n\n")
   uod = 1
}


function usertime()

# function to calculate a users time during the time period recorded
# in the activity.log
# syntax:  mtk M=UT u="%N" c=%G activity.log
{
   if (ut_setup != 1)
      ut_header()

   if ($0 ~ u)  {

      start = $3
      if (1 > getline) exit
         stop = $2
         if (substr(start,1,2) == substr(stop,1,2)) {
            time_this_call = (stop - start)
            total_time += time_this_call       # time processing is arranged
            num_calls += 1                     # in order of most to least
            printf("")                        # common
            }
         else if (substr(start,1,2) < substr(stop,1,2)) {
            time_this_call = ((stop - start) - 40)
            total_time += time_this_call
            num_calls += 1                     # printed symbols indicate
            printf("")                        # occurances
            }
         else if (substr(start,1,2) > substr(stop,1,2)) {
            time_this_call = ((60 - substr(start,3,2) + substr(stop,3,2)))
            total_time += time_this_call
            num_calls += 1
            printf("")
            }
         }
      }
END   {
      if (ut_flag != 1) {

         printf("%s", BRWHITE)

         if (total_time == 0 && num_calls == 0) {
            printf("\nNo time accumulated from previous calls.\n\n")
            exit
         }
         else {
            printf("\nTotal number of calls:  %d\n", num_calls)
            printf("Total time in minutes:  %d\n", total_time)

            if (total_time >= 60)
               printf("Total time in hours:    %.2f\n", total_time/60)

               printf("Average time per call:  %.2f minutes\n\n", total_time/num_calls)
               printf("%s", DEFAULT)
         }
      }
}


function ut_header()

# function called by usertime() to generate header information
{
   printf("Processing ")
   ftrans_flag = 1
   ut_setup = 1
}


function popular_files()

# function to generate a four column list of the most popular downloaded
# files.  used in conjunction with a sorted sldir list.
# syntax:  mtk M=PF bbs=%S x=<number to list> c=%G d=Y (or N) <filename.lst>
{
   get_date()
   if (pf_setup != 1)
      pf_header()
   ut_flag = 1
   ftrans_flag = 1
   {
   if (i < x )
      {
      if ((($3 + 0) > 0) && (($3 + 0) ~ /^[0-9]/))
         {
         gsub(/\*|\+/, "")                         # strip SL file symbols
         printf("   %-14s%3s", $1, $3)             # for column output
         getline; i++; if (($3 != 0) && (i < x))
         gsub(/\*|\+/, "")
         printf("  %-14s%3s",  $1, $3)
         getline; i++; if (($3 != 0) && (i < x))
         gsub(/\*|\+/, "")
         printf("  %-14s%3s",  $1, $3)
         getline; i++; if (($3 != 0) && (i < x))
         gsub(/\*|\+/, "")
         printf("  %-14s%3s\n",  $1, $3)
         i++
         }
       else if (($3 + 0) !~ /^[0-9]/)            # pass over any non-numeric
         getline                                 # material
       }
   else
      exit
   }
}


function pf_header()

# function called by popular_files() to generate header information
{
   pf_setup = 1
   printf("\n")
   printf(" %s BBS Top %d Downloaded Files\n", bbs, x)
   if (d == "Y")
      printf(" As compiled on %s\n\n", whole_date)
   else
      printf("\n")
}

 
function relister()

# function to modify a list created by sldir
# syntax:  mtk M=RL x=<string> <filename.lst> ><newlist.lst>
{
   get_date()
   ut_flag = 1
   ftrans_flag = 1
   color_flag = 1
   {
   if ($0 ~ "SLDIR Directory") {
      printf("Directory Listing for %s\n%s\n\n", whole_date, x)
      getline
      }
   else if ($1 == "Filename") {
      print $0
      getline
      getline
      print "------------------------------------------------------------------------------"
      }
   else if ($0 ~ /Searching/)                 # replace "searching" with
      printf("File Area %s %s\n\n", $2, $3)   # "file area"
   }

   {
   if ($1 != "Searching")
   {
      if (($2 == "+") || ($2 == "*"))
         printf("%-12s %-1s %6s  %3s %9s  %s\n",  $1, $2, $3, $4, $5, substr($0, 39, 40))
      else
         printf("%-14s %6s  %3s %9s  %s\n",  $1, $2, $3, $4, substr($0, 39, 40))
   }
  }
}


function color()

# function to interpret users ansi selection.  default color is bright white
# if ansi is selected
{
   if (color_flag != 1) {
      color_flag = 1

   if (c != "N") {
      BRWHITE = "[1;37m"          # ansi code for bright white
      DEFAULT = "[0m"             # ansi code for default colors
   }
   else  {
      BRWHITE = ""                 # no ansi codes for NONE
      DEFAULT = ""
   }
   printf("%s", BRWHITE)
   printf("\n\n")
   }
}


function get_date()

# function to get the current time and date from dos
{
   date = ctime()

   day_of_week = substr(date, 1, 3)

   { weekdays["Sun"] = "Sunday"    }         # expand day abbreviations
   { weekdays["Mon"] = "Monday"    }         # to full name
   { weekdays["Tue"] = "Tuesday"   }
   { weekdays["Wed"] = "Wednesday" }
   { weekdays["Thu"] = "Thursday"  }
   { weekdays["Fri"] = "Friday"    }
   { weekdays["Sat"] = "Saturday"  }

   if (day_of_week in weekdays)
      today = weekdays[day_of_week]

   month_of_year = substr(date, 5, 3)

   { monthyear["Jan"] = "January"   }        # expand month abbreviations
   { monthyear["Feb"] = "February"  }        # to full month
   { monthyear["Mar"] = "March"     }
   { monthyear["Apr"] = "April"     }
   { monthyear["May"] = "May"       }
   { monthyear["Jun"] = "June"      }
   { monthyear["Jul"] = "July"      }
   { monthyear["Aug"] = "August"    }
   { monthyear["Sep"] = "September" }
   { monthyear["Oct"] = "October"   }
   { monthyear["Nov"] = "November"  }
   { monthyear["Dec"] = "December"  }

   if (month_of_year in monthyear)
      month = monthyear[month_of_year]

   hour = (substr(date,12,2) + 0)     # expands time and determines
                                      # am or pm
   if (hour < 12)
      time_of_day = sprintf("%d:%s am", substr(date,12,2), substr(date,15,2))
   else if (hour == 12)
      time_of_day = sprintf("%d:%s pm", substr(date,12,2), substr(date,15,2))
   else if (hour > 12)
      time_of_day = sprintf("%d:%s pm", (substr(date,12,2) - 12), substr(date,15,2))

   whole_date = sprintf("%s, %s %d, %s - %s", today, month, \
                        substr(date, 9, 2), substr(date, 21, 4), time_of_day)
}

# eof
